#ifndef __PPA_STACK_AL_H__20081103_1153__
#define __PPA_STACK_AL_H__20081103_1153__



/******************************************************************************
**
** FILE NAME    : ppa_stack_al.h
** PROJECT      : PPA
** MODULES      : PPA Protocol Stack Adaption Layer (Linux)
**
** DATE         : 3 NOV 2008
** AUTHOR       : Xu Liang
** DESCRIPTION  : PPA Protocol Stack Adaption Layer (Linux) Header File
** COPYRIGHT    :              Copyright (c) 2009
**                          Lantiq Deutschland GmbH
**                   Am Campeon 3; 85579 Neubiberg, Germany
**
**   For licensing information, see the file 'LICENSE' in the root folder of
**   this software module.
**
** HISTORY
** $Date        $Author         $Comment
** 03 NOV 2008  Xu Liang        Initiate Version
*******************************************************************************/

/*! \file ppa_stack_al.h
    \brief This file contains es.
                provide linux os depenent api for PPA to use
*/



  #include <linux/ppa_api_common.h>


#if defined(CONFIG_LTQ_PMCU) || defined(CONFIG_LTQ_PMCU_MODULE) || defined(CONFIG_LTQ_CPU_FREQ)
  //PMCU specific Head File
#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33)
  #include <asm/ifx/types.h>
  #include <asm/ifx/pmcu.h>
#else
  //#include <types.h>
  #include <cpufreq/ltq_cpufreq.h>
//  #include <asm/mach-ltqcpe/ifx_pmcu.h>
#endif

#endif  //end of CONFIG_LTQ_PMCU

#if defined(CONFIG_LTQ_DATAPATH) && CONFIG_LTQ_DATAPATH
#include <net/datapath_api.h>
#endif

#ifdef NO_DOXY
 
typedef enum { 
	LTQ_MEI_UNKNOWN = 0, 
	/** To register function for getting showtime status. */
	LTQ_MEI_SHOWTIME_CHECK = 1, 
	/** To register function for showtime entry signalling. */
	LTQ_MEI_SHOWTIME_ENTER = 2, 
	/** To register function for showtime exit signalling. */
	LTQ_MEI_SHOWTIME_EXIT = 3,
	/** To register function for TC-Layer selection.
	    For DSL/PP switchover concept (also refer to DSLCPE_SW-858) */ 
	LTQ_MEI_TC_REQUEST = 4,
	/** To register function for performing reset of TC-Layer.
	    (also refer to PPA_SYS-353) */ 
	LTQ_MEI_TC_RESET = 5
} e_ltq_mei_cb_type; 

/**
	This definition is used by the MEI Driver within TC request trigger to
	inform the PPA about the TC-Layer that has been negotiated during handshake
	and therefore needs to be loaded/configured on the PPA side. */
typedef enum { 
	/** Request to disable the TC */
	MEI_TC_REQUEST_OFF = 0,
	/** Request to load/configure the PTM (EFM) TC */
	MEI_TC_REQUEST_PTM = 1,
	/** Request to load/configure the ATM TC */
	MEI_TC_REQUEST_ATM = 2,
	/** Delimiter only! */
	MEI_TC_REQUEST_LAST = 3
} mei_tc_request_type; 

/**
	This bit-field definition is used by the MEI Driver to trigger reset
	handling(s) within TC-Layer of the PPA. */
typedef enum { 
	/** This value is defined for initialization purpose only.
		 At least one of the defined bits of the bitmask shall be set. */
	MEI_TC_RESET_CLEAN = 0x00000000,
	/** Request to perform a reset of the TC codeword buffer */
	MEI_TC_RESET_CW_BUFFER = 0x00000001
} mei_tc_reset_type;

struct ltq_mei_atm_showtime_info {
	void *check_ptr;
	void *enter_ptr;
	void *exit_ptr;
    void *req_tc_ptr;
    void *tc_reset_ptr;
};

/** Function pointer for new DSL/PP switchover concept(also refer to DSLCPE_SW-858)
   \param line            Line number
   \param tc_reqest_type  TC-Layer to be used/configured
   \param is_bonding      Defines whether bonding is used (>=1) or not (0)
	\return                0 if successful
 */
typedef int (*mei_tc_request_t)(
	const unsigned char line,
	mei_tc_request_type tc_reqest_type,
	int is_bonding);

/** Function pointer for triggering a reset of the TC (also refer to PPA_SYS-353)
   \param line            Line number 
	\param tc_reset_type   TC Reset type to be performed 
	\return                0 if successful */ 
typedef int (*mei_tc_reset_t)(
	const unsigned char,
	mei_tc_reset_type tc_reset_type);
 
#endif
/*
 * ####################################
 *              Definition
 * ####################################
 */

/*! \def PPA_ETH_ALEN
    \brief Macro that specifies the maximum length of an Ethernet MAC address.
 */
#define PPA_ETH_ALEN                            ETH_ALEN

/*! \def PPA_ETH_HLEN
    \brief Macro that specifies the maximum length of an Ethernet MAC header.
 */
#define PPA_ETH_HLEN                            ETH_HLEN

/*! \def PPA_ETH_CRCLEN
    \brief Macro that specifies the maximum length of an Ethernet CRC.
 */
#define PPA_ETH_CRCLEN                          4

/*! \def PPA_IF_NAME_SIZE
    \brief Macro that specifies the maximum size of one interface name
 */
#define PPA_IF_NAME_SIZE                        IFNAMSIZ

/*! \def PPA_IF_SUB_NAME_MAX_NUM
        \brief Macro that specifies the maximum size of one interface name
     */
#define PPA_IF_SUB_NAME_MAX_NUM                5

#define PPA_IF_SUB_MAX_NUM	               16


/*! \def PPA_IPPROTO_TCP
    \brief Macro that specifies TCP flag
 */
#define PPA_IPPROTO_TCP                         6

/*! \def PPA_IPPROTO_UDP
    \brief Macro that specifies UDP flag
 */
#define PPA_IPPROTO_UDP                         17

/*! \def PPA_USER
    \brief Macro that specifies the flag for the buffer type from User Space via ioctl
 */
#define  PPA_USER                                       __user

/*
 *  definition for application layer
 */
/*! \def ETH_ALEN
    \brief Macro that specifies the maximum length of an Ethernet MAC address.
 */
  #define ETH_ALEN                              6

/*! \def IFNAMSIZ
    \brief Macro that specifies the maximum size of an interface NAME
 */
  #define IFNAMSIZ                              16

#undef NIPQUAD
/*! \def NIPQUAD
    \brief Macro that specifies NIPQUAD definition for printing IPV4 address
 */
#define NIPQUAD(addr) \
    ((unsigned char *)&addr)[0], \
    ((unsigned char *)&addr)[1], \
    ((unsigned char *)&addr)[2], \
    ((unsigned char *)&addr)[3]

#undef NIPQUAD_FMT
/*! \def NIPQUAD_FMT
    \brief Macro that specifies NIPQUAD_FMT format definition for printing IPV4 address
 */
#define NIPQUAD_FMT "%u.%u.%u.%u"


#undef NIP6
/*! \def NIP6
    \brief Macro that specifies NIP6 definition for printing IPV6 address
 */
#define NIP6(addr) \
         ntohs(((unsigned short *)addr)[0]), \
         ntohs(((unsigned short *)addr)[1]), \
         ntohs(((unsigned short *)addr)[2]), \
         ntohs(((unsigned short *)addr)[3]), \
         ntohs(((unsigned short *)addr)[4]), \
         ntohs(((unsigned short *)addr)[5]), \
         ntohs(((unsigned short *)addr)[6]), \
         ntohs(((unsigned short *)addr)[7])

#undef NIP6_FMT
/*! \def NIP6_FMT
    \brief Macro that specifies NIP6_FMT format definition for printing IPV6 address
 */
#define NIP6_FMT "%04x:%04x:%04x:%04x:%04x:%04x:%04x:%04x"


/*
 * ####################################
 *              Data Type
 * ####################################
 */

/*
 *  data type for application layer
 */
#ifndef BUILD_FROM_PPA_ADAPT
/*!
    \brief This is the unsigned char 32-bit data type.
*/
  typedef unsigned long         uint32_t;

/*!
    \brief This is the unsigned char 16-bit data type.
*/
  typedef unsigned short        uint16_t;

/*!
    \brief This is the unsigned char 8-bit data type.
*/
  typedef unsigned char         uint8_t;

/*!
    \brief This is the unsigned char 64-bit data type.
*/
  typedef unsigned long long        uint64_t;

#endif
/*
 *  data type for API
 */
/*!
    \brief  Pointer to interface name
*/
typedef char                    PPA_IFNAME;
/*!
    \brief  This is the data structure holding the IP address. It helps to provide future compatibility for IPv6 support.
              Currently it only supports IPv4.

*/
typedef uint32_t                IPADDR;

/*!
    \brief Union of PPA network address
 */
  typedef union {
	uint32_t ip;        /*!< the storage buffer for ipv4 */
#ifdef CONFIG_LTQ_PPA_IPv6_ENABLE
	uint32_t ip6[4];    /*!< the storage buffer for ipv6 */
#endif
  }PPA_IPADDR;




/*
 * ####################################
 *           Inline Functions
 * ####################################
 */



/*
 * ####################################
 *             Declaration
 * ####################################
 */
/** \addtogroup  PPA_ADAPTATION_LAYER */
/*@{*/

/* @} */


#endif  //  __IFX_PPA_STACK_AL_H__20081103_1153__
